﻿<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
  version="2.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  xmlns:my="myFunctions">

<!-- 
<purpose>transform XCQL to xpath
! WARNING: not pure Xpath! actually it adapts the query on the query-interface of MD-Repository
in particular there is special handling for index "collection" 
</purpose>
<history>
	<change on="2010-01-28" type="created" by="vr"></change>			
	<change on="2010-07-01" type="changed" by="vr">handling of collection-filter</change>
</history>
<sample>
<triple>
  <boolean>
    <value>and</value>
  </boolean>
  <leftOperand>
    <searchClause>
      <index>bath.author</index>
      <relation>
        <value>any</value>
      </relation>
      <term>fish</term>
    </searchClause>
  </leftOperand>
  <rightOperand>
    <searchClause>
      <index>dc.title</index>
      <relation>
        <value>all</value>
      </relation>
      <term>cat dog</term>
    </searchClause>
  </rightOperand>
</triple>
</sample>
      -->
      

<xsl:include href="cmd_functions.xsl"/>
<!--<xsl:include href="params.xsl"/>
<xsl:variable name="terms_setup" select="document($terms_setup_uri)" />
<xsl:variable name="terms_flat" select="document($terms_flat_uri)" />
-->
<xsl:output method="text" />

	
<xsl:template match="triple">

	<xsl:choose>
	<!--  this is rather hacky yet
aim is: special handling of collection-index (as separate parameter)
precondition: collection-index just as rightOperand on top-nesting-level and boolean='and'
 -->
		<xsl:when test="rightOperand/searchClause/index='collection'" >
			<xsl:apply-templates select="leftOperand/*" />
			<xsl:apply-templates select="rightOperand/*" mode="collection"/>			
		</xsl:when>
		<xsl:otherwise>
			<xsl:if test="not(ancestor::triple)" >//CMD/*[</xsl:if>			
				<xsl:call-template name="boolean" >
					<xsl:with-param name="op" select="boolean/value" />
					<xsl:with-param name="left" >
						<xsl:apply-templates select="leftOperand/*" />
					</xsl:with-param>
					<xsl:with-param name="right" >
						<xsl:apply-templates select="rightOperand/*" />
					</xsl:with-param>
				</xsl:call-template>
			<xsl:if test="not(ancestor::triple)" >]</xsl:if>				
		</xsl:otherwise>
	</xsl:choose>
	
</xsl:template>

<xsl:template name="boolean">
	<xsl:param name="op" />
	<xsl:param name="left" />
	<xsl:param name="right" />
	<xsl:message>XCQL2XPath.template-boolean.$op:<xsl:value-of select="$op" /></xsl:message>
	

	<xsl:choose>
		<xsl:when test="$op='and'">
			<xsl:text></xsl:text>
			<xsl:value-of select="$left" />
			<xsl:text>][</xsl:text><xsl:value-of select="$right" /><xsl:text></xsl:text>
		</xsl:when>
		<xsl:when test="$op='or'">
			<xsl:value-of select="$left" />
			<xsl:text>%20or%20</xsl:text><xsl:value-of select="$right" />
		</xsl:when>		
	</xsl:choose>
	
	
</xsl:template>

<xsl:template match="searchClause">
	<xsl:apply-templates select="index" />
	<xsl:apply-templates select="relation">
		<xsl:with-param name="term" select="term" />
	</xsl:apply-templates>	
</xsl:template>


<!--   aim is: special handling of collection-filter (as separate parameter) -->
<xsl:template match="searchClause" mode="collection">
		<xsl:text>&amp;collection=</xsl:text><xsl:value-of select="term"></xsl:value-of>		
</xsl:template>

<xsl:template match="index">

	<!-- reverting the "escaping" of whitespaces in indices with datcat-name -->
	<xsl:variable name="ix_string" select="replace(text(),'_',' ')" />
	
	<xsl:variable name="ix_resolved" >	
		<xsl:choose>
			<xsl:when test="contains(.,':')">
				<xsl:variable name="prefix" select="substring-before(.,':')" />				
				<xsl:variable name="termset" select="$terms_setup/Termsets/Termset[@id=$prefix]" />				
				
				<xsl:message>XCQL2XPath.index:<xsl:copy-of select="$termset"/>
								ix_string:<xsl:value-of select="$ix_string"/>
				</xsl:message>
				<xsl:choose>
				<!-- magic happening here  -->
					<xsl:when test="$termset[@type='dcr' or @type='rr']">
						<xsl:variable name="expanded_context" select="$terms_flat//context[@path=$ix_string]"></xsl:variable>
							<xsl:variable name="expanded_query" >
								<xsl:text>(</xsl:text>
								<xsl:for-each select="distinct-values($expanded_context//context[@elem]/@path)">
									<xsl:variable name="prefix" select="substring-before(.,':')" />
									<xsl:variable name="termset" select="$terms_setup/Termsets/Termset[@id=$prefix]" />									
									<xsl:value-of select="concat($termset/@name,'//',substring-after(.,':'))" />
									<xsl:if test="position()!=last()"><xsl:text>|</xsl:text></xsl:if>
								</xsl:for-each>
								<xsl:text>)</xsl:text>
							</xsl:variable>
							<xsl:message>expanded-context:<xsl:value-of select="$expanded_query" /></xsl:message>
						<xsl:value-of select="$expanded_query" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="concat($termset/@name,'//',substring-after(.,':'))" />	
					</xsl:otherwise>
				</xsl:choose>
				 
			</xsl:when>
			<xsl:otherwise><xsl:value-of select="." />
			</xsl:otherwise>
		</xsl:choose>		
		
	</xsl:variable>
	<!-- <xsl:variable name="ix_xpathed" select="translate($ix_resolved, '.', '/')" ></xsl:variable> -->
	<xsl:variable name="ix_xpathed" select="my:index2xpath(.)" ></xsl:variable> 
	<xsl:choose>
		<xsl:when test=".='cql.serverChoice' or .='*'" >//Components</xsl:when>				
		<xsl:when test="ancestor::triple" >
			<xsl:value-of select="concat('.//',$ix_xpathed)" />
		</xsl:when>
		<xsl:otherwise>		
			<xsl:value-of select="concat('//',$ix_xpathed)" />	
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="relation">
	<xsl:param name="term" />
	
	<xsl:text>[</xsl:text>
		<xsl:choose>			
			<xsl:when test="value='contains' or preceding-sibling::index='cql.serverChoice'" >
				<xsl:text>contains(.,'</xsl:text><xsl:value-of select="$term"/><xsl:text>')</xsl:text>				
			</xsl:when>
			<xsl:when test="value='any'" >
			<!--  use full-text querying: [ft:query(., "language")] -->
				<xsl:text>ft:query(.,'</xsl:text><xsl:value-of select="$term"/><xsl:text>')</xsl:text>
									
			</xsl:when>
			<xsl:otherwise>		
				<xsl:text>.</xsl:text> <xsl:value-of select="value" />
				<xsl:text>'</xsl:text><xsl:value-of select="$term"/><xsl:text>'</xsl:text>	
			</xsl:otherwise>
		</xsl:choose>	
	<xsl:text>]</xsl:text>
	
</xsl:template>


</xsl:stylesheet>